#!/usr/bin/env python
from argparse import ArgumentParser

from .smart.parser import date_parse
from .output import MODES


def main():
    parser = ArgumentParser('timezone translation CLI')

    parser.add_argument(
            'time',
            nargs='*',
            help='standard or semantic date string',
            )

    parser.add_argument(
            '--mode',
            type=str,
            help='set parse mode : (compare)',
            default='compare',
            )

    parser.add_argument(
            '--timezones',
            type=str,
            help='comma-separated list of timezones for output',
            default=None,
            )

    args = parser.parse_args()

    MODES[args.mode](
            timezone = date_parse(*args.time),
            timezones = args.timezones,
            )


if __name__=='__main__':
    main()



from .tzinfo import TZOFFSETS


def shift(timezone, count):
    '''
    returns the timezone-aware datetime shifted [+count] (1hr) timezones
    '''
    offset = timezone.utcoffset().total_seconds()
    new_offset = (((offset/60/60 + (count)) + 11) % 24) - 11

    new_tz = TZOFFSETS[new_offset]
    return timezone.astimezone(new_tz)



from datetime import datetime

from dateutil.tz import gettz
from pytz import common_timezones, all_timezones, timezone


def get_local_timezone_name():
    return datetime.now().astimezone().tzname()

ALIAS_DICT = {
        'US/Central': [
            'CT', 'CDT', 'CST',
            'AL', 'AR', 'IA', 'IL', 'KS',
            'LA', 'MN', 'MO', 'MS', 'ND',
            'NE', 'OK', 'SD', 'TN', 'TX',
            'WI',
            ],
        'US/Eastern': [
            'ET', 'EDT', 'EST',
            'CT', 'DC', 'DE', 'FL', 'GA',
            'IN', 'MA', 'MD', 'ME', 'MI',
            'NC', 'NH', 'NJ', 'NY', 'OH',
            'PA', 'RI', 'SC', 'VA', 'VT',
            'WV',
            ],
        'US/Mountain': [
            'MT', 'MDT', 'MST',
            'AZ', 'CO', 'ID', 'MT', 'NM',
            'UT', 'WY',
            ],
        'US/Pacific':  [
            'PT', 'PDT', 'PST',
            'WA', 'OR', 'NV', 'CA',
            ],
        }

REVERSE_ALIAS_DICT = {
        alias: name
        for name, aliases in ALIAS_DICT.items()
        for alias in aliases
        }

TZINFOS = {
        **{
            name : gettz(name)
            for name in all_timezones
            },
        **{
            alias.upper() : gettz(name)
            for name, aliases in ALIAS_DICT.items()
            for alias in aliases
            },
        }

TZOFFSETS = {
        datetime.now(timezone(name)).utcoffset().total_seconds()/60/60: gettz(name)
        for name in common_timezones
        }



#!/usr/bin/env python
from setuptools import setup, find_packages

from tzfzf import __version__, __author__


with open('README.md', 'r', encoding='utf-8') as readme:
    long_description = readme.read()
    long_description_content_type = 'text/markdown'

setup(
    name='tzfzf',
    version=__version__,
    description='a terminal CLI / API for quick cross-timezone lookups',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/wrynegade/tzfzf',
    author=__author__,
    author_email='yage@yage.io',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'python-dateutil',
        'pytz',
        'rich',
        ],
    classifiers=[
        'Programming Language :: Python :: 3 :: Only',
        ],
    entry_points={
        'console_scripts': [
            'tzfzf = tzfzf.cli:main',
            ],
        },
    )



from pytz import utc
from rich import print as rich_print
from rich.panel import Panel
from rich.align import Align


from ..smart.math import shift
from ..smart.tzinfo import REVERSE_ALIAS_DICT, get_local_timezone_name

def compare(timezone, timezones=None):
    '''
    compare indicated datetime to local timezone ±3 hours
    '''
    panels = [
            get_panel(tz)
            for tz in [shift(timezone, x) for x in range(-3, 4)]
            ]

    panels.append(get_panel(timezone.astimezone(utc)))


    for p in panels:
        rich_print(p)

def get_panel(tz):
    return Panel(
            Align.center(tz.strftime('%I:%M %p')),
            title = get_name(tz),
            )

def get_name(tz):
    tzname = tz.tzname()

    name = REVERSE_ALIAS_DICT.get(tzname, tzname)

    if tzname == get_local_timezone_name():
        color = 'green'
    elif tzname == 'UTC':
        color = 'yellow'
    else:
        color = 'red'

    return f'[{color}]{name}[/{color}]'



from datetime import datetime

from dateutil.parser import parse, ParserError
from pytz import timezone

from .exceptions import DateParseError
from .tzinfo import TZINFOS


def date_parse(*args, attempts=0, MAX_ATTEMPTS=6):
    '''
    parses a timezone-sensitive date from a variety of standard
    and semantic datetime strings

    with no arguments, gets the current, UTC time
    '''
    if len(args) == 0:
        return datetime.utcnow().replace(tzinfo=timezone('UTC'))

    try:
        d = parse(' '.join(args).upper(), tzinfos=TZINFOS)
        if d.tzinfo is not None and d.tzinfo.utcoffset(d) is not None:
            return d
        return d.astimezone()

    except ParserError as e:
        if attempts > MAX_ATTEMPTS:
            raise DateParseError() from e

        func = _rotate if attempts < MAX_ATTEMPTS else _unzip

        return date_parse(
                *func(args),
                attempts = attempts + 1,
                MAX_ATTEMPTS = min(MAX_ATTEMPTS, len(args)),
                )

def _rotate(args):
    return [*args[1::], args[0]]

def _unzip(args):
    return [*args[1::2], *args[0::2]]
